##############################################################################
# TAG HEBREW AND GREEK LANGUAGE IN ASCII FILES
##############################################################################
#
# (c) 2004 www.SpiritAndTruth.org
# May be freely used without restriction.
# mailto:contact@SpiritAndTruth.org
#
#

HELP = '''
Usage: langtag [flags] path1 [path2...]

FLAGS
-----

-?          Print out help and exit.

-help       Print out help and exit.

-advance    Normally, breathmarks are only advanced to precede
            capital vowels. When the -advance flag is specified,
            breathmarks for all vowels and diphthongs are advanced
            to precede the vowel or diphthong.

-space      When advancing breathmarks (see -advance), insert
            an extra space ahead of the advanced breathmark to
            increase separation from the preceding text.

-unicode    Use unicode fonts (see below).

TYPICAL USAGE
-------------
The most common way which langtag is used is to tag HTML 
files containing Hebrew or Greek language represented as ASCII
characters and convert them for display using HTML markup.

    Greek           {grk|Ijerosovluma}
    Hebrew          {heb|yis.r'ael}
    Transliteration {tra|agapao-}

Each markup consists of three components: 

  (1) a "tag" identifying the markup
  (2) a vertical bar separating the tag from the contents
  (3) text representing the contents of the markup to be converted

  NOTE: THERE CANNOT BE A SPACE BETWEEN THE END OF THE TAG (1) AND
  THE VERTICAL BAR (2). This is to differentiate between langtag
  markup and the frequent use of the vertical bar to separate text
  links as in:

      choice A | choice B | choice C

The following markup tags are supported:

    grk     Greek using codes corresponding to the BSTGreek font.

    heb     Hebrew using codes corresponding to the BSTHebrew font.

    grk+    Same as grk, but include a transliteration of Greek.

    heb+    Same as heb, but include a transliteration of Hebrew.

    tra     Transliteration (see font_example_langtag.htm for examples).

    [other] All other tags are assumed to represent HTML entities and
            are expanded as:

                <other>text</other>

            For example:

                {sup|superscript text}

            Expands to:

                <sup>superscript text</sup>

Markup tags may be nested. For example:

    {h1|My Heading with {i|Italics}}
    {p|This is a paragraph containing {i|italic text} and {b|bold text}.}

They may also include attributes:

    {font size="+2" color="red"|large red text}

which generates:

    <font size="+1" color="red">large red text</font>

HEBREW AND GREEK
----------------
The codes used for the Hebrew and Greek input text correspond to the free
BSTHebrew and BSTGreek fonts available from

    http://www.SpiritAndTruth.org/fonts/index.htm

The Hebrew or Greek markup is automatically converted as follows:

1. Final characters (e.g, sigma in Greek, mem in Hebrew) are
   converted to the appropriate form. (This means you can type a
   final 's' or 'm' in your language and let langtag take
   care of converting it to the alternative final form.)

2. Appropriate <span> markers are placed around the text.
   For example:

        <span class="greek">Ijerosuvluma</span> (Jerusalem)
        <span class="hebrew">yis.r'ael</span> (Israel)

   The user must use an associated style (usually from a style sheet)
   to associate a font and other display characteristics with each
   language (see below).

3. If the -unicode flag is NOT specified, then Hebrew text is
   automatically reversed:

        <span class="hebrew">lea'r.siy</span> (Israel)

4. If the -unicode flag is specified, then the order of the Hebrew text
   is not modified. Applications, such as Web browsers, which support unicode
   automatically reverse the order of Hebrew when they display the text.

   In addition, the -unicode flag causes the class names within the <span>
   markers designating Greek and Hebrew to include a "u" prefix:

        <span class="ugreek">Ijerosuvluma</span> (Jerusalem)
        <span class="uhebrew">yis.r'ael</span> (Israel)

    This allows the same style sheet to specify different fonts for use with
    non-unicode vs. unicode fonts.

    You can use the included style sheet by placing the following line
    in the header portion of your HTML file:

        <link rel=stylesheet type="text/css" HREF="styles.css">


    Or you can specify the styles in the <head> portion of the document:

        <head>
            ...
            <style> 
            <!-- 
            .ugreek
                {
                font-family:BibliaLS;
                color:blue;
                font-size:130%;
                }
            .greek
                {
                font-family:BSTGreek;
                color:blue;
                font-size:130%;
                }
            .uhebrew
                {
                font-family:BibliaLS;
                color:blue;
                font-size:150%;
                }
            .hebrew
                {
                font-family:BSTHebrew;
                color:blue;
                font-size:150%;
                }
            .translit
                {
                color:green;
                font-style:italic;
                }
            -->
            </style> 
        </head>

    See the associated styles.css file as an example.

CONVERTING FILES
----------------
A command-line (DOS) utility named LANGTAG.EXE is included. To convert one or
more files, this utility can be run at the command line to specify the flags
and files (and/or folders) to convert. Alternately, if the default operation
is adequate, you can drag and drop files to convert them:

1. Within Explorer, drag-and-drop the input file(s) onto the LANGTAG.EXE
   file in the directory where LANGTAG.EXE was unzipped.

2. LANGTAG.EXE will create a corresponding output file in the same 
   folder as the original input file, but with a different name:

   Dropping 'MyFile.htm' will create 'MyFile_langtag.htm' as the translated
   output file.

Developed by: www.SpiritAndTruth.org contact@SpiritAndTruth.org
'''

VERSION = '0.5'
HISTORY = '''

HISTORY

    0.5     Indicates location of output file.

    0.4     Removed support for Hebrew and Greek tagging of the form
            {h |. . . and {g |. . . in favor of the more explicit for 
            {heb| . . .  and {grk| . . . .

            Added support for including attributes in tags, for example:

                {font size="+2" color="red"|large red text}

            which generates:

                <font size="+1" color="red">large red text</font>

    0.3     Added support for new markup tags: grk+, heb+, tra,
            and generic HTML.

    0.2     Minor corrections.

    0.1     Initial version.
'''

import hebgrk
import os.path
import sys
import string
import re

##############################################################################
# Visit a directory when walking a directory tree.
##############################################################################
def visit( processor, dirname, names ) :
    # Visit each directory below a directory tree.

    #print 'Visiting directory:', dirname
    for name in names :
        processor.scanFile( os.path.join( dirname, name ) )

    return


##############################################################################
# Identify and tag language markup within files.
##############################################################################
class Processor :


    def __init__( self, args ) :
        self.m_me = args.pop( 0 )
        self.m_tagPat = '{(?P<id>(?P<first>[^\|\{\s]+)(?P<others>\s+[^\|\{]+)?)\|(?P<text>[^}{]+)}'
        self.m_tagExp = re.compile( self.m_tagPat, re.IGNORECASE )
        self.m_langPat = '((?P<id>heb|grk|tra)(?P<plus>\+)?\s*)'
        self.m_langExp = re.compile( self.m_langPat, re.IGNORECASE )
        self.m_unicode = 0
        self.m_advance = 0
        self.m_space = 0
        self.m_spanFormatGreek = '<span class="greek">%s</span>'
        self.m_spanFormatHebrew = '<span class="hebrew">%s</span>'
        self.m_spanFormatTranslit = '<span class="translit">%s</span>'
        self.m_spanFormatTranslitAppend = ' [<span class="translit">%s</span>]'
        self.m_spanFormatGenericTag = '<%s%s>%s</%s>'

        # Help requested?
        try :
            idx = args.remove( '-?' )
            print 'langtag', VERSION
            print 'hebgrk', hebgrk.VERSION
            print '---------------------------------------------'
            print '\tlangtag'
            print '---------------------------------------------'
            print HELP
            print HISTORY
            print '---------------------------------------------'
            print '\thebgrk'
            print '---------------------------------------------'
            print hebgrk.HISTORY
            sys.exit( 1 )
        except ValueError : pass
        try :
            idx = args.remove( '-help' )
            print VERSION
            print HELP
            print HISTORY
            sys.exit( 1 )
        except ValueError : pass

        # Unicode specified?
        try :
            idx = args.remove( '-unicode' )
            self.m_unicode = 1
            print 'Using unicode'
            self.m_spanFormatGreek = '<span class="ugreek">%s</span>'
            self.m_spanFormatHebrew = '<span class="uhebrew">%s</span>'
        except ValueError : pass

        # Advance breath marks?
        try :
            idx = args.remove( '-advance' )
            self.m_advance = 1
            print 'Advancing breathmarks'
        except ValueError : pass

        # Space when advance breathmarks?
        try :
            idx = args.remove( '-space' )
            self.m_space = 1
        except ValueError : pass

        # For each name to be scanned for tags.
        for path in args :

            # Walk directories.
            if os.path.isdir( path ) : 
                os.path.walk( path, visit, self )

            # Scan individual files.
            elif os.path.isfile( path ) : 
                self.scanFile( path )


    def scanFile( self, path ) :
        # Scan file and tag for language.
        inFile = None
        outFile = None

        try :

            # Open the file for read.
            print
            print 'Reading', path
            inFile = open( path, 'r' )

            # Open a corresponding output file.
            file, ext = os.path.splitext( path )
            outPath = file + '_langtag' + ext
            print 'Writing', outPath
            outFile = open( outPath, 'w' )

            # For each input line.
            output = ''
            if 0 :
                for line in inFile.readlines() :
                    output = self.translateLine( line )

            # Join lines so that auto-wrap doesn't break tags.
            else :
                oneLine = string.join( inFile.readlines(), '' )
                output = self.translateLine( oneLine )

            print 'Writing output to %s' % outPath
            outFile.write( output )

        finally :
            if inFile : inFile.close()
            if outFile : outFile.close()


    def translateLine( self, line ) :
        # Translate a line of language markup.

        # While another translation match was found.
        # (Finds nested translations which, after expansion,
        # may lead to other outer translations matching.)
        input = line
        more = 1
        while more :
            more = 0

            # While another translation found.
            tagMatch = 1
            output = ''
            remainder = input
            while tagMatch :

                # If found another markup in the line.
                tagMatch = self.m_tagExp.search( remainder )
                if tagMatch :

                    # At least one translation is being performed.
                    more = 1

                    # Pass through portion up to the markup.
                    output += remainder[ 0 : tagMatch.start() ]

                    # Get the markup text.
                    tagId = tagMatch.group( 'id' )
                    tagText = tagMatch.group( 'text' )
                    print 'Tag:', tagId, tagText
                    firstId = tagMatch.group( 'first' )
                    othersId = tagMatch.group( 'others' )

                    # Is this a language-specific markup?
                    langMatch = self.m_langExp.match( firstId )
                    if langMatch :

                        # Is a transliteration to be appended?
                        langId = langMatch.group( 'id' )
                        doTranslit = langMatch.group( 'plus' )
                        translit = ''

                        # If it is Hebrew.
                        if 'heb' == langId :


                            #Convert finals.
                            hebrew = hebgrk.hebrewConvertFinals( tagText )

                            # Perform transliteration?
                            if doTranslit :
                                translit = hebgrk.translitBSTHebrew( tagText )
                                translit = hebgrk.translitToUnicodeHtml( translit )

                            # If using unicode.
                            if self.m_unicode :

                                # Convert to unicode, application will take care of reverse display.
                                hebrew = hebgrk.BSTHebrewToUnicodeHtml( hebrew, 0, not self.m_unicode )

                            # Else not using unicode.
                            else :

                                # Reverse Hebrew text.
                                hebrew = list( hebrew )
                                hebrew.reverse()
                                hebrew = string.join( hebrew, '' )

                            # Tag with <span>
                            output += self.m_spanFormatHebrew % hebrew

                            # Tag transliteration, if any.
                            if translit : output += self.m_spanFormatTranslitAppend % translit

                        # Else if it is Greek.
                        elif 'grk' == langId :

                            # Convert finals.
                            greek = hebgrk.greekConvertFinals( tagText )

                            # Perform transliteration (before move breathmarks)?
                            if doTranslit :
                                translit = hebgrk.translitBSTGreek( greek )
                                translit = hebgrk.translitToUnicodeHtml( translit )

                            # Advance breathmarks?
                            greek = hebgrk.advanceBreathMarkCapital( greek, self.m_space )
                            if self.m_advance :
                                greek = hebgrk.advanceBreathMark( greek, self.m_space )

                            # Translate Greek to unicode?
                            if self.m_unicode : 
                                greek = hebgrk.BSTGreekToUnicodeHtml( greek, extended_codes = 1 )

                            # Tag with <span>
                            output += self.m_spanFormatGreek % greek

                            # Tag transliteration, if any.
                            if translit : output += self.m_spanFormatTranslitAppend % translit

                        # Else if manual transliteration.
                        elif 'tra' == langId :

                            # Transliterate to HTML codes.
                            translit = hebgrk.translitToUnicodeHtml( tagText )
                            output += self.m_spanFormatTranslit % translit

                        # Else unknown language tag
                        else :
                            msg = 'ERROR - unknown language tag: %s' % firstId
                            assert 0, msg

                    # Else not language-specific markup.
                    else :

                        # Do generic tag conversion.
                        if not othersId : othersId = ''
                        output += self.m_spanFormatGenericTag % ( firstId, othersId, tagText, firstId )

                    # Move past the tag markup.
                    remainder = remainder[ tagMatch.end() : ]

            # Append remainder to output.
            output = output + remainder

            # Get ready to search for nested markup.
            input = output

        return output

# If module is launched as a program, perform tagging using specified files.
if __name__ == '__main__' :
    tag = Processor( sys.argv )
    
